/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

package org.ow2.petals.cli.shell;

import java.io.IOException;

import jline.TerminalFactory;
import jline.console.ConsoleReader;

import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.exception.NoInteractiveShellException;
import org.ow2.petals.cli.api.exception.ShellException;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.shell.exception.UnknownCommandException;

public class PetalsInteractiveCli extends AbstractShell {

    /**
     * The response 'yes' to enter on confirmation
     */
    private final static String YES = "y";

    /**
     * The response 'no' to enter on confirmation
     */
    private final static String NO = "n";

    /**
     * Character massking password
     */
    private final static char PASSWORD_MASK = '*';

    ConsoleReader cr = null;

    /**
     * 
     * @param isDebugEnabled
     * @param isYesFlagEnable
     *            Flag to enable the automatic confirmation
     * @throws IOException
     */
    public PetalsInteractiveCli(final boolean isDebugEnabled, final boolean isYesFlagEnable)
            throws IOException {
        super(System.out, System.err, isDebugEnabled, isYesFlagEnable);
        this.cr = new ConsoleReader(System.in, System.out, TerminalFactory.create());
        this.cr.setHistoryEnabled(true);
        // Because of a problem in JLine about escape characters ('\\') of
        // regexp, event designators
        // (http://www.gnu.org/software/bash/manual/html_node/Event-Designators.html)
        // support is disabled.
        this.cr.setExpandEvents(false);
        this.setPrompt(Shell.DEFAULT_PROMPT);
    }

    @Override
    public void run() {
        try {

            // init completion
            CommandCompleter commandCompleter = new CommandCompleter(this.getCommands());
            this.cr.addCompleter(commandCompleter);

            String line;
            while (this.isCommandRead && (line = this.cr.readLine()) != null) {
                try {
                    this.evaluate(line);
                } catch (final CommandInvalidException ce) {
                    this.printCommandSyntaxError(ce);
                } catch (CommandException ce) {
                    this.printCommandExecutionError(ce);
                } catch (UnknownCommandException uce) {
                    // No stack trace even in debug mode
                    this.errStream.println("ERROR: " + uce.getMessage());
                } finally {
                    this.cr.flush();
                }
            }
        } catch (IOException ioe) {
            this.printError(ioe);
            this.setExitStatus(1);
        }
    }

    @Override
    public void setPrompt(final String prompt) {
        this.cr.setPrompt(prompt);
    }

    @Override
    public String askQuestion(String question, boolean isReplyPassword)
            throws NoInteractiveShellException, ShellException {

        if (this.isYesFlagEnabled) {
            throw new NoInteractiveShellException();
        } else {
            try {
                this.cr.setHistoryEnabled(false);
                final String prompt = this.cr.getPrompt();
                final String response;
                if (isReplyPassword) {
                    response = this.cr.readLine(question, new Character(PASSWORD_MASK));
                } else {
                    response = this.cr.readLine(question);
                }
                this.cr.setPrompt(prompt);
                this.cr.setHistoryEnabled(true);
                return response;
            } catch (final IOException e) {
                throw new ShellException(e);
            }
        }
    }

    /**
     * {@inheritDoc Shell}
     * 
     * <p>
     * Special notes for the implementation {@link PetalsInteractiveCli}: if the
     * 'yes' flag is not enabled, the message is displayed, and the user is
     * invited to reply by 'y' or 'n'.
     * </p>
     */
    @Override
    public boolean confirms(final String message) throws ShellException {
        if (!super.confirms(message)) {
            String confirmationResponse;
            do {
                confirmationResponse = this.askQuestion(message, false);
            } while (!confirmationResponse.equals(YES) && !confirmationResponse.equals(NO));

            return confirmationResponse.equals(YES);
        }
        return true;
    }

    @Override
    public boolean isInteractive() {
        return true;
    }

}
