/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/
package org.ow2.petals.cli.shell;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.ParseException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.api.command.CommandRegistration;
import org.ow2.petals.cli.api.exception.DuplicatedCommandException;
import org.ow2.petals.cli.shell.exception.ShellCreationException;

/**
 * Shell factory creating the right shell according to provided on command line.
 * @author Christophe DENEUX - Linagora
 */
public class ShellFactory {

    /**
     * The short name of the option enabling debug information
     */
    public final static String DEBUG_SHORT_OPTION = "d";

    /**
     * The long name of the option enabling debug information
     */
    public final static String DEBUG_LONG_OPTION = "debug";

    /**
     * The short name of the 'yes' flag option
     */
    public final static String YESFLAG_SHORT_OPTION = "y";

    /**
     * The long name of the 'yes' flag option
     */
    public final static String YESFLAG_LONG_OPTION = "yes";

    /**
     * The short name of the option executing a command from the command line
     */
    public final static String COMMAND_SHORT_OPTION = "c";

    /**
     * The short name of the option enabling the mode 'console'
     */
    public final static String CONSOLE_SHORT_OPTION = "C";

    /**
     * The short name of the option enabling the file execution
     */
    public final static String FILE_SHORT_OPTION = null;

    /**
     * The long name of the option enabling the file execution
     */
    public final static String FILE_LONG_OPTION = "file";

    private final static String USAGE = "[--" + FILE_LONG_OPTION + "] <filename> | -"
            + COMMAND_SHORT_OPTION + " -- <command> | -" + CONSOLE_SHORT_OPTION + "]";

    public static final ShellFactory INSTANCE = new ShellFactory();

    private ShellFactory() {
        // NOP
    }

    public static ShellFactory getInstance() {
        return INSTANCE;
    }

    public static void addShellOptions(final OptionGroup optionsGrp) {
        optionsGrp.addOption(OptionBuilder.withLongOpt("console").hasArg(false)
                .withDescription("Enable the mode 'console'.").create(CONSOLE_SHORT_OPTION));
        optionsGrp
                .addOption(OptionBuilder
                        .withArgName("filename")
                        .withLongOpt(FILE_LONG_OPTION)
                        .hasArg()
                        .withDescription(
                                "Enable the script file execution. If filename is '-', commands are read from the stdin.")
                        .create(FILE_SHORT_OPTION));
        optionsGrp.addOption(OptionBuilder.withLongOpt("command")
                .withDescription("Execute a command given on the command line after '--'.")
                .create(COMMAND_SHORT_OPTION));

    }

    public static String getUsage() {
        return USAGE;
    }

    /**
     * <p>
     * Create a new shell from arguments of the command line:
     * <ul>
     * <li>- <b>"-c"</b>: command execution on command line ({@link PetalsCli},</li>
     * <li>- <b>"-C"</b>: interactive mode ({@link PetalsInteractiveCli},</li>
     * <li>- <b>"--file"</b>: file mode ({@link PetalsScriptShell},</li>
     * </ul>
     * </p>
     *
     * @param cmdLine
     * @param args
     * @return
     * @throws DuplicatedCommandException
     *             A command has been registered twice. As it's a hard coded
     *             registration, it's a <b>BIG</b> bug.
     * @throws ParseException
     *             An error occurs about provided options or arguments.
     * @throws FileNotFoundException
     *             This error is thrown by file mode when the file containing
     *             commands does not exist.
     * @throws ShellCreationException
     *             An error occurs creating the shell
     */
    public AbstractShell newShell(final CommandLine cmdLine, final String[] args)
            throws DuplicatedCommandException, ParseException, ShellCreationException {

        final AbstractShell shell;
        if (cmdLine.hasOption(COMMAND_SHORT_OPTION)) {
            final String[] commandArgs = cmdLine.getArgs();
            if (commandArgs.length == 0) {
                throw new ParseException(Constants.CommandMessages.MISSING_COMMAND);
            } else {
                shell = new PetalsCli(commandArgs, cmdLine.hasOption(DEBUG_SHORT_OPTION),
                        cmdLine.hasOption(YESFLAG_SHORT_OPTION));
            }
        } else if (cmdLine.hasOption(CONSOLE_SHORT_OPTION)) {
            try {
                shell = new PetalsInteractiveCli(cmdLine.hasOption(DEBUG_SHORT_OPTION),
                        cmdLine.hasOption(YESFLAG_SHORT_OPTION));
            } catch (final IOException e) {
                throw new ShellCreationException(e);
            }
        } else if (cmdLine.hasOption(FILE_LONG_OPTION)) {
            shell = new PetalsFileScriptShell(cmdLine.getOptionValue(FILE_LONG_OPTION),
                    cmdLine.getArgs(), cmdLine.hasOption(DEBUG_SHORT_OPTION),
                    cmdLine.hasOption(YESFLAG_SHORT_OPTION));
        } else {
            // File mode
            final String[] remainingArgs = cmdLine.getArgs();
            if (remainingArgs.length > 0) {
                final String filename = remainingArgs[0];
                if ("-".equals(filename)) {
                    // Stdin mode
                    shell = new PetalsInlinedScriptShell(remainingArgs,
                            cmdLine.hasOption(DEBUG_SHORT_OPTION),
                            cmdLine.hasOption(YESFLAG_SHORT_OPTION));
                } else {
                    // File mode
                    shell = new PetalsFileScriptShell(filename, remainingArgs,
                            cmdLine.hasOption(DEBUG_SHORT_OPTION),
                            cmdLine.hasOption(YESFLAG_SHORT_OPTION));
                }
            } else {
                // A file name is missing
                shell = null;
            }
        }

        if (shell != null) {
            CommandRegistration.registers(shell);
        }

        return shell;

    }
}
