/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

package org.ow2.petals.cli.shell.command;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import jline.console.completer.StringsCompleter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.AdminFactory;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

import com.ebmwebsourcing.easycommons.lang.CollectionHelper;
import com.ebmwebsourcing.easycommons.lang.CollectionHelper.Filter;

/**
 * This command list JBI artifacts deployed on petals ESB.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class ListArtifacts extends AbstractCommand {

    static final String TYPE_SHORT_OPTION = "t";

    static final String TYPE_LONG_OPTION = "type";

    private static final Option TYPE_OPTION = OptionBuilder
            .isRequired(false)
            .withLongOpt(TYPE_LONG_OPTION)
            .withDescription(
                    "<artifact-type> is one of the following optional values: SL, BC, SE, SA, SU, used to restrict the returned list.")
            .hasArgs(1).withArgName("artifact-type").create(TYPE_SHORT_OPTION);

    static final String REGEX_SHORT_OPTION = "p";

    static final String REGEX_LONG_OPTION = "pattern";

    private static final Option REGEX_OPTION = OptionBuilder
            .isRequired(false)
            .withLongOpt(REGEX_LONG_OPTION)
            .withDescription(
                    "<artifact-pattern> is an optional RegExp pattern to filter the content of the returned list. All the returned JBI artifacts must have a JBI identifier matching the pattern.")
            .hasArgs(1).withArgName("artifact-pattern").create(REGEX_SHORT_OPTION);

    private final String separator = " ";

    /**
     *
     */
    final Comparator<Artifact> artifactNameComparator = new Comparator<Artifact>() {
        @Override
        public int compare(Artifact o1, Artifact o2) {
            return o1.getName().compareTo(o2.getName());
        }
    };

    private final PosixParser clParser;

    public ListArtifacts() {
        super("list");
        this.clParser = new PosixParser();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("List JBI artifacts name and current status");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        this.completers.put(TYPE_SHORT_OPTION, new StringsCompleter(SharedLibrary.TYPE,
                ComponentType.BC.toString(), ComponentType.SE.toString(), ServiceAssembly.TYPE,
                ServiceUnit.TYPE));
    }

    @Override
    public void execute(String[] args) throws CommandException {
        try {
            ArrayList<Filter<Artifact>> filters = new ArrayList<Filter<Artifact>>();
            CommandLine cli = this.clParser.parse(getOptions(), args);
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            if (cli.hasOption(TYPE_SHORT_OPTION)) {
                filters.add(new TypeFilter(cli.getOptionValue(TYPE_SHORT_OPTION)));
            }
            if (cli.hasOption(REGEX_SHORT_OPTION)) {
                filters.add(new RegexFilter(cli.getOptionValue(REGEX_SHORT_OPTION)));
            }

            ArtifactAdministration artifactAdministration = AdminFactory.newInstance()
                    .createArtifactAdministration();
            List<Artifact> artifacts = artifactAdministration.listArtifacts();

            StringBuilder out = new StringBuilder();
            for (Artifact artifact : sortArtifacts(artifacts, filters)) {
                out.append(artifact.getName());
                if (artifact instanceof ArtifactState) {
                    out.append(this.separator);
                    out.append(((ArtifactState) artifact).getState().toString());
                }
                out.append(this.separator);
                out.append(artifact.getType());
                out.append("\n");
            }
            if (!artifacts.isEmpty()) {
                this.getShell().getPrintStream().print(out);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ArtifactAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    /**
     *
     * @param artifacts
     * @return
     */
    private ArrayList<Artifact> sortArtifacts(List<Artifact> artifacts,
            List<Filter<Artifact>> filters) {
        Collection<SharedLibrary> sls = new TreeSet<SharedLibrary>(this.artifactNameComparator);
        Collection<Component> bcs = new TreeSet<Component>(this.artifactNameComparator);
        Collection<Component> ses = new TreeSet<Component>(this.artifactNameComparator);
        Collection<ServiceAssembly> sas = new TreeSet<ServiceAssembly>(this.artifactNameComparator);
        Collection<ServiceUnit> sus = new TreeSet<ServiceUnit>(this.artifactNameComparator);

        for (Artifact artifact : artifacts) {
            if (artifact instanceof SharedLibrary) {
                sls.add((SharedLibrary) artifact);
            } else if (ComponentType.BC.toString().equals(artifact.getType())) {
                bcs.add((Component) artifact);
            } else if (ComponentType.SE.toString().equals(artifact.getType())) {
                ses.add((Component) artifact);
            } else if (artifact instanceof ServiceAssembly) {
                sas.add((ServiceAssembly) artifact);
                sus.addAll(((ServiceAssembly) artifact).getServiceUnits());
            }
        }

        ArrayList<Artifact> sortedArtifacts = new ArrayList<Artifact>(sls);
        sortedArtifacts.addAll(bcs);
        sortedArtifacts.addAll(ses);
        sortedArtifacts.addAll(sas);
        sortedArtifacts.addAll(sus);

        if (filters != null && filters.size() > 0)
            CollectionHelper.filter(sortedArtifacts, filters);

        return sortedArtifacts;
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        options.addOption(TYPE_OPTION);
        options.addOption(REGEX_OPTION);
        return options;
    }

    private class TypeFilter implements Filter<Artifact> {
        private final String type;

        public TypeFilter(String type) {
            this.type = type;
        }

        @Override
		public boolean accept(Artifact a) {
            if (this.type == null) {
                return true;
            } else if (a.getType().equals(this.type)) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * Implements strict regex filter
     *
     * @author "Mathieu CARROLLE - EBM WebSourcing"
     *
     */
    private class RegexFilter implements Filter<Artifact> {
        private final Pattern pattern;

        public RegexFilter(String regex) {
            this.pattern = Pattern.compile(regex);
        }

        @Override
		public boolean accept(Artifact a) {
            Matcher m = this.pattern.matcher(a.getName());
            return m.matches();
        }
    }

}
