/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

package org.ow2.petals.cli.shell.command;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import jline.console.completer.Completer;
import jline.console.completer.StringsCompleter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.AdminFactory;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.artifact.Logger;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.InvalidLogLevelException;
import org.ow2.petals.admin.api.exception.LoggerNotFoundException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class LoggerSet extends AbstractCommand {

    static final String LOGGER_NAME_SHORT_OPTION = "n";

    static final String LOGGER_NAME_LONG_OPTION = "logger";

    private static final Option LOGGER_NAME_OPTION = OptionBuilder.isRequired(true)
            .withLongOpt(LOGGER_NAME_LONG_OPTION)
            .withDescription("The logger name for which the level will be set.").hasArgs(1)
            .withArgName("logger-name").create(LOGGER_NAME_SHORT_OPTION);

    static final String LEVEL_SHORT_OPTION = "l";

    static final String LEVEL_LONG_OPTION = "level";

    private static final Option LEVEL_OPTION = OptionBuilder.isRequired(true)
            .withLongOpt(LEVEL_LONG_OPTION).withDescription("The level to set to the logger name.")
            .hasArgs(1).withArgName("logger-level").create(LEVEL_SHORT_OPTION);

    static final String RESULT_SUCCEEDED = "The log level was succesfully updated.";

    private final CommandLineParser clParser;

    public LoggerSet() {
        super("logger-set");
        this.clParser = new PosixParser();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Set the specified level to the specified logger");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public void execute(String[] args) throws CommandException {

        try {
            final CommandLine cli = this.clParser.parse(getOptions(), args, false);
            if (cli.hasOption(LOGGER_NAME_SHORT_OPTION) && cli.hasOption(LEVEL_SHORT_OPTION)) {
                // TODO: The check about log level could be remove when the JMX
                // API of the container will manage it correctly (returning an
                // exception if the log level is not supported)
                final String level = cli.getOptionValue(LEVEL_SHORT_OPTION);
                if (!"SEVERE".equals(level) && !"WARNING".equals(level) && !"INFO".equals(level)
                        && !"CONFIG".equals(level) && !"MONIT".equals(level)
                        && !"FINE".equals(level) && !"FINER".equals(level)
                        && !"FINEST".equals(level)) {
                    throw new InvalidLogLevelException(level);
                }

                final ContainerAdministration containerAdministration = AdminFactory.newInstance()
                        .createContainerAdministration();

                // TODO: The check about logger could be remove when the JMX
                // API of the container will manage it correctly (returning an
                // exception if the logger does not exist)
                final String loggerName = cli.getOptionValue(LOGGER_NAME_SHORT_OPTION);
                final List<Logger> loggers = containerAdministration.getLoggers();
                boolean found = false;
                for (final Logger logger : loggers) {
                    if (logger.getName().equals(loggerName)) {
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    throw new LoggerNotFoundException(loggerName);
                }
                containerAdministration.changeLoggerLevel(loggerName, level);
                this.getShell().getPrintStream().println(RESULT_SUCCEEDED);

            } else {
                throw new CommandBadArgumentNumberException(this);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final InvalidLogLevelException e) {
            throw new CommandInvalidArgumentException(this, LEVEL_OPTION, e.getLevel());
        } catch (final LoggerNotFoundException e) {
            throw new CommandInvalidArgumentException(this, LOGGER_NAME_OPTION, e.getLogger());
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        try {
            ContainerAdministration containerAdministration = AdminFactory.newInstance()
                    .createContainerAdministration();
            final List<String> loggerNames = new LinkedList<String>();
            final List<Logger> loggers = containerAdministration.getLoggers();
            for (final Logger logger : loggers) {
                loggerNames.add(logger.getName());
            }

            this.completers.put(LOGGER_NAME_SHORT_OPTION, new StringsCompleter(loggerNames));
        } catch (final ContainerAdministrationException e) {
            // no available completion
        }

        this.completers.put(LEVEL_SHORT_OPTION, new StringsCompleter("SEVERE", "WARNING", "INFO",
                "CONFIG", "MONIT", "FINE", "FINER", "FINEST"));
        return this.completers;
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        options.addOption(LOGGER_NAME_OPTION);
        options.addOption(LEVEL_OPTION);
        return options;
    }

}
