/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/

package org.ow2.petals.cli.shell.command;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.AdminFactory;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.exception.ShellException;
import org.ow2.petals.cli.api.shell.Shell;

/**
 * This command stop a petals ESB container.
 * 
 * @author Sebastien Andre - EBM WebSourcing
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class Stop extends AbstractCommand {

    /**
     * The shutdown option
     */
    public static final String SHUTDOWN_SHORT_OPTION = "s";
    private static final String SHUTDOWN_LONG_OPTION = "shutdown";
    private static final Option SHUTDOWN_OPTION = OptionBuilder
            .hasArg(false)
            .withLongOpt(SHUTDOWN_LONG_OPTION)
            .withDescription(
                    "A flag to skip confirmation.")
            .create(SHUTDOWN_SHORT_OPTION);
    
    /**
     * The confirmation skip option
     */
    public final static String YESFLAG_SHORT_OPTION = "y";

    private static final Option YESFLAG_OPTION = OptionBuilder.hasArg(false)
            .withDescription("A flag to skip confirmation.").create(YESFLAG_SHORT_OPTION);

    public final static String SHUTDOWN_CONFIRMATION_MSG = "Are you sure you want to shutdown the container? (y/n) ";

    public Stop() {
        super();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Stop the container");
    }

    @Override
    public void execute(final String[] args) throws CommandException {
        try {
            final CommandLineParser parser = new PosixParser();
            final CommandLine cmd = parser.parse(this.getOptions(), args);

            final Shell shell = this.getShell();
            if (!cmd.hasOption(SHUTDOWN_SHORT_OPTION)) {
                if (cmd.hasOption(YESFLAG_SHORT_OPTION)) {
                    throw new CommandInvalidException(this, String.format(
                            Constants.CommandMessages.UNRECOGNIZED_OPTION, YESFLAG_SHORT_OPTION));
                }

                this.stopContainer();

            } else {
                if (!cmd.hasOption(YESFLAG_SHORT_OPTION)) {
                    if (shell.confirms(SHUTDOWN_CONFIRMATION_MSG)) {
                        this.shutdownContainer();
                    } else if (!shell.isInteractive()) {
                        throw new CommandMissingYesFlagException(this, YESFLAG_OPTION);
                    }
                } else {
                    this.shutdownContainer();
                }
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final ShellException e) {
            throw new CommandException(this, e);
        }
    }

    /**
     * Stop the container on which we are connected
     * 
     * @throws ContainerAdministrationException
     *             An error occurs during the stop
     */
    private void stopContainer() throws ContainerAdministrationException {

        final ContainerAdministration containerAdministration = AdminFactory.newInstance()
                .createContainerAdministration();
        containerAdministration.stopContainer();

        this.getShell().setPrompt(Shell.DEFAULT_PROMPT);
    }

    /**
     * Shutdown the container on which we are connected
     * 
     * @throws ContainerAdministrationException
     *             An error occurs during the shutdown
     */
    private void shutdownContainer() throws ContainerAdministrationException {
        final ContainerAdministration containerAdministration = AdminFactory.newInstance()
                .createContainerAdministration();
        containerAdministration.shutdownContainer();

        this.getShell().setPrompt(Shell.DEFAULT_PROMPT);
    }

    @Override
    public Options getOptions() {
        Options options = new Options();
        options.addOption(YESFLAG_OPTION);
        options.addOption(SHUTDOWN_OPTION);
        return options;
    }
}
