/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

package org.ow2.petals.cli.shell.command;

import static org.ow2.petals.cli.Constants.CommonOption.ARTIFACT_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.ARTIFACT_SHORT_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import jline.console.completer.Completer;
import jline.console.completer.FileNameCompleter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.AdminFactory;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.shell.PetalsInteractiveCli;
import org.ow2.petals.cli.shell.util.Utils;

/**
 * This command stop an artifact.
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class StopArtifact extends AbstractCommand {

    private final ArtifactAdministration artifactAdministration;

    public StopArtifact() {
        super("stop-artifact");
        this.artifactAdministration = AdminFactory.newInstance().createArtifactAdministration();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Stop a JBI artifact");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        this.completers.put(URL_SHORT_OPTION, new FileNameCompleter());
    }

    @Override
    public void execute(String[] args) throws CommandException {

        CommandLineParser parser = new PosixParser();
        try {
            CommandLine cl = parser.parse(getOptions(), args, true);
            if (cl.hasOption(URL_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 2)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                URL artifactURL = Utils.toURL(cl.getOptionValue(URL_SHORT_OPTION));
                this.artifactAdministration.stopArtifact(artifactURL);
            } else if (cl.hasOption(ARTIFACT_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 3)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                String[] values = cl.getOptionValues(ARTIFACT_SHORT_OPTION);
                if (values.length == 1) {
                    this.artifactAdministration.stopArtifact(null, values[0]);
                } else {
                    this.artifactAdministration.stopArtifact(values[1], values[0]);
                }
            } else {
                throw new CommandBadArgumentNumberException(this);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ArtifactAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final MalformedURLException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        OptionGroup op = new OptionGroup();
        op.addOption(ARTIFACT_OPTION);
        op.addOption(URL_OPTION);
        options.addOptionGroup(op);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        Utils.addArtifactTypeAndNameCompleter((PetalsInteractiveCli) getShell(),
                this.artifactAdministration, this.completers);
        return this.completers;
    }
}
