/****************************************************************************
 *
 * Copyright (c) 2010-2012, EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

package org.ow2.petals.cli.shell.command;

import static org.ow2.petals.cli.Constants.CommonOption.ARTIFACT_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.ARTIFACT_SHORT_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import jline.console.completer.Completer;
import jline.console.completer.FileNameCompleter;

import org.apache.commons.cli.AlreadySelectedException;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.AdminFactory;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.exception.ShellException;
import org.ow2.petals.cli.shell.PetalsInteractiveCli;
import org.ow2.petals.cli.shell.util.Utils;

/**
 * This command undeploy an artifact from a petals esb.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class Undeploy extends AbstractCommand {

    /**
     * The bulk option
     */
    public final static String BULK_SHORT_OPTION = Constants.CommonOption.BULK_SHORT_OPTION;

    private final static String BULK_LONG_OPTION = Constants.CommonOption.BULK_LONG_OPTION;

    protected static final Option BULK_OPTION = OptionBuilder.withLongOpt(BULK_LONG_OPTION)
            .hasOptionalArg().withArgName("artifacts-directory")
            .withDescription("Stop and uninstall or undeploy JBI artifacts")
            .create(BULK_SHORT_OPTION);

    /**
     * The confirmation skip option
     */
    public final static String YESFLAG_SHORT_OPTION = "y";

    private static final Option YESFLAG_OPTION = OptionBuilder.hasArg(false)
            .withDescription("A flag to skip confirmation.").create(YESFLAG_SHORT_OPTION);

    public final static String UNDEPLOY_ALL_CONFIRMATION_MSG = "Are you sure you want to undeploy all artifacts? (y/n) ";

    private final ArtifactAdministration artifactAdministration;

    public Undeploy() {
        super();
        this.artifactAdministration = AdminFactory.newInstance().createArtifactAdministration();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Stop and uninstall or undeploy JBI artifacts");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        this.completers.put(URL_SHORT_OPTION, new FileNameCompleter());
    }

    @Override
    public void execute(final String[] args) throws CommandException {
        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cl = parser.parse(getOptions(), args, true);
            if (cl.hasOption(BULK_SHORT_OPTION) && !cl.hasOption(URL_SHORT_OPTION)
                    && !cl.hasOption(ARTIFACT_SHORT_OPTION)) {
                final String directory = cl.getOptionValue(BULK_SHORT_OPTION);
                if (directory != null && !directory.isEmpty()) {
                    final URL url = Utils.toURL(directory);
                    if (url.getProtocol().equals("file")) {
                        final File filePath = new File(url.getFile());
                        this.artifactAdministration.stopAndUndeployAllArtifacts(Utils
                                .getUrls(filePath));
                    } else {
                        throw new CommandException(this,
                                "Directory undeployment is only allowed for file protocol");
                    }
                } else {
                    // Undeploy all artifacts of the container
                    if (!cl.hasOption(YESFLAG_SHORT_OPTION)) {
                        if (this.getShell().confirms(UNDEPLOY_ALL_CONFIRMATION_MSG)) {
                            this.artifactAdministration.stopAndUndeployAllArtifacts(null);
                        } else if (!this.getShell().isInteractive()) {
                            throw new CommandMissingYesFlagException(this, YESFLAG_OPTION);
                        }
                    } else {
                        this.artifactAdministration.stopAndUndeployAllArtifacts(null);
                    }
                }
            } else if (cl.hasOption(URL_SHORT_OPTION) && !cl.hasOption(BULK_SHORT_OPTION)
                    && !cl.hasOption(ARTIFACT_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2)) {
                    throw new CommandBadArgumentNumberException(this);
                }
                for (String arg : cl.getOptionValues(URL_SHORT_OPTION)) {
                    this.artifactAdministration.stopAndUndeployArtifact(Utils.toURL(arg));
                }
            } else if (cl.hasOption(ARTIFACT_SHORT_OPTION) && !cl.hasOption(URL_SHORT_OPTION)
                    && !cl.hasOption(BULK_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 3)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                String[] values = cl.getOptionValues(ARTIFACT_SHORT_OPTION);
                if (values.length == 1) {
                    this.artifactAdministration.stopAndUndeployArtifact(null, values[0]);
                } else {
                    this.artifactAdministration.stopAndUndeployArtifact(values[1], values[0]);
                }
            } else {
                throw new CommandBadArgumentNumberException(this);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final AlreadySelectedException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ArtifactAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final MalformedURLException e) {
            throw new CommandException(this, e);
        } catch (final ShellException e) {
            throw new CommandException(this, e);
        } catch (IOException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        OptionGroup op = new OptionGroup();
        op.addOption(BULK_OPTION);
        op.addOption(ARTIFACT_OPTION);
        op.addOption(URL_OPTION);
        options.addOptionGroup(op);
        options.addOption(YESFLAG_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        Utils.addArtifactTypeAndNameCompleter((PetalsInteractiveCli) getShell(),
                this.artifactAdministration, this.completers);
        return this.completers;
    }
}
